=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-06-18 20:04:16 +0000
+++ bzrlib/builtins.py	2007-06-19 14:49:06 +0000
@@ -2551,13 +2551,13 @@
     
     @display_command
     def run(self, branch, other):
-        from bzrlib.revision import MultipleRevisionSources
+        from bzrlib.revision import ensure_null, MultipleRevisionSources
         
         branch1 = Branch.open_containing(branch)[0]
         branch2 = Branch.open_containing(other)[0]
 
-        last1 = branch1.last_revision()
-        last2 = branch2.last_revision()
+        last1 = ensure_null(branch1.last_revision())
+        last2 = ensure_null(branch2.last_revision())
 
         graph = branch1.repository.get_graph(branch2.repository)
         base_rev_id = graph.find_unique_lca(last1, last2)
@@ -3538,6 +3538,7 @@
 
     def run(self, submit_branch=None, public_branch=None, patch_type='bundle',
             sign=False, revision=None, mail_to=None, message=None):
+        from bzrlib.revision import ensure_null, NULL_REVISION
         if patch_type == 'plain':
             patch_type = None
         branch = Branch.open('.')
@@ -3568,6 +3569,9 @@
                 revision_id = revision[0].in_history(branch).rev_id
         else:
             revision_id = branch.last_revision()
+        revision_id = ensure_null(revision_id)
+        if revision_id == NULL_REVISION:
+            raise errors.BzrCommandError('No revisions to bundle.')
         directive = merge_directive.MergeDirective.from_objects(
             branch.repository, revision_id, time.time(),
             osutils.local_time_offset(), submit_branch,

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2007-06-08 21:48:42 +0000
+++ bzrlib/bundle/commands.py	2007-06-19 14:49:06 +0000
@@ -27,6 +27,7 @@
 from bzrlib import (
     branch,
     errors,
+    revision as _mod_revision,
     urlutils,
     )
 """)
@@ -156,7 +157,8 @@
                                            base_branch.last_revision())
             graph = target_branch.repository.get_graph()
             base_revision = graph.find_unique_lca(
-                base_branch.last_revision(), target_revision)
+                _mod_revision.ensure_null(base_branch.last_revision()),
+                _mod_revision.ensure_null(target_revision))
 
         if output is not None:
             fileobj = file(output, 'wb')

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2007-06-18 20:02:18 +0000
+++ bzrlib/graph.py	2007-06-19 14:49:06 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib import errors
 from bzrlib.deprecated_graph import (node_distances, select_farthest)
 from bzrlib.revision import NULL_REVISION
 
@@ -46,6 +47,9 @@
     def __init__(self, parent_providers):
         self._parent_providers = parent_providers
 
+    def __repr__(self):
+        return "_StackedParentsProvider(%r)" % self._parent_providers
+
     def get_parents(self, revision_ids):
         """Find revision ids of the parents of a list of revisions
 
@@ -88,6 +92,10 @@
             conforming to the behavior of StackedParentsProvider.get_parents
         """
         self.get_parents = parents_provider.get_parents
+        self.parents_provider = parents_provider
+
+    def __repr__(self):
+        return 'Graph(%r)' % self.parents_provider
 
     def find_lca(self, *revisions):
         """Determine the lowest common ancestors of the provided revisions
@@ -149,6 +157,8 @@
         This allows calculation of graph difference from the results of this
         operation.
         """
+        if None in revisions:
+            raise errors.InvalidRevisionId(None, self)
         common_searcher = self._make_breadth_first_searcher([])
         common_ancestors = set()
         searchers = [self._make_breadth_first_searcher([r])

=== modified file 'bzrlib/merge_directive.py'
--- bzrlib/merge_directive.py	2007-06-08 21:48:42 +0000
+++ bzrlib/merge_directive.py	2007-06-19 14:49:06 +0000
@@ -202,7 +202,10 @@
         If the message is not supplied, the message from revision_id will be
         used for the commit.
         """
-        t = testament.StrictTestament3.from_revision(repository, revision_id)
+        t_revision_id = revision_id
+        if revision_id == 'null:':
+            t_revision_id = None
+        t = testament.StrictTestament3.from_revision(repository, t_revision_id)
         submit_branch = _mod_branch.Branch.open(target_branch)
         if submit_branch.get_public_branch() is not None:
             target_branch = submit_branch.get_public_branch()
@@ -210,6 +213,7 @@
             patch = None
         else:
             submit_revision_id = submit_branch.last_revision()
+            submit_revision_id = _mod_revision.ensure_null(submit_revision_id)
             repository.fetch(submit_branch.repository, submit_revision_id)
             graph = repository.get_graph()
             ancestor_id = graph.find_unique_lca(revision_id,

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2007-06-08 21:48:42 +0000
+++ bzrlib/repofmt/knitrepo.py	2007-06-19 14:49:06 +0000
@@ -44,6 +44,9 @@
     def __init__(self, knit):
         self._knit = knit
 
+    def __repr__(self):
+        return 'KnitParentsProvider(%r)' % self._knit
+
     def get_parents(self, revision_ids):
         parents_list = []
         for revision_id in revision_ids:

=== modified file 'bzrlib/tests/blackbox/test_bundle.py'
--- bzrlib/tests/blackbox/test_bundle.py	2007-03-30 02:36:36 +0000
+++ bzrlib/tests/blackbox/test_bundle.py	2007-06-19 14:49:06 +0000
@@ -21,10 +21,10 @@
 
 from bzrlib.bundle.serializer import read_bundle
 from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCaseInTempDir
-
-
-class TestBundle(TestCaseInTempDir):
+from bzrlib import tests
+
+
+class TestBundle(tests.TestCaseWithTransport):
 
     def make_trees(self):
         grandparent_tree = BzrDir.create_standalone_workingtree('grandparent')
@@ -92,3 +92,9 @@
         stdout = self.run_bzr_subprocess('bundle')[0]
         br = read_bundle(StringIO(stdout))
         self.assertRevisions(br, ['revision3'])
+
+    def test_no_common_ancestor(self):
+        foo = self.make_branch_and_tree('foo')
+        bar = self.make_branch_and_tree('bar')
+        os.chdir('foo')
+        self.run_bzr('bundle', '../bar')

=== modified file 'bzrlib/tests/blackbox/test_find_merge_base.py'
--- bzrlib/tests/blackbox/test_find_merge_base.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_find_merge_base.py	2007-06-19 14:49:06 +0000
@@ -32,3 +32,9 @@
         r = self.run_bzr('find-merge-base', '.', '../a', backtick=True)
         self.assertEqual(q, r)
         
+    def test_find_null_merge_base(self):
+        tree = self.make_branch_and_tree('foo')
+        tree.commit('message')
+        tree2 = self.make_branch_and_tree('bar')
+        r = self.run_bzr('find-merge-base', 'foo', 'bar')[0]
+        self.assertEqual('merge base is revision null:\n', r)

=== modified file 'bzrlib/tests/blackbox/test_merge_directive.py'
--- bzrlib/tests/blackbox/test_merge_directive.py	2007-04-01 03:58:12 +0000
+++ bzrlib/tests/blackbox/test_merge_directive.py	2007-06-19 14:49:06 +0000
@@ -213,3 +213,17 @@
                                   '.')
         call = connect_calls[0]
         self.assertEqual(('bogushost', 0), call[1:3])
+
+    def test_no_common_ancestor(self):
+        foo = self.make_branch_and_tree('foo')
+        foo.commit('rev1')
+        bar = self.make_branch_and_tree('bar')
+        os.chdir('foo')
+        self.run_bzr('merge-directive', '../bar')
+
+    def test_no_commits(self):
+        foo = self.make_branch_and_tree('foo')
+        bar = self.make_branch_and_tree('bar')
+        os.chdir('foo')
+        self.run_bzr_error(('No revisions to bundle.', ),
+                            'merge-directive', '../bar')

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2007-06-18 20:02:18 +0000
+++ bzrlib/tests/test_graph.py	2007-06-19 14:49:06 +0000
@@ -14,7 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib import graph
+from bzrlib import (
+    errors,
+    graph,
+    )
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCaseWithMemoryTransport
 
@@ -170,6 +173,7 @@
         ancestry_1 should always have a single common ancestor
         """
         graph = self.make_graph(ancestry_1)
+        self.assertRaises(errors.InvalidRevisionId, graph.find_lca, None)
         self.assertEqual(set([NULL_REVISION]),
                          graph.find_lca(NULL_REVISION, NULL_REVISION))
         self.assertEqual(set([NULL_REVISION]),

=== modified file 'bzrlib/tests/test_merge_directive.py'
--- bzrlib/tests/test_merge_directive.py	2007-04-19 07:17:16 +0000
+++ bzrlib/tests/test_merge_directive.py	2007-06-19 14:49:06 +0000
@@ -202,6 +202,13 @@
         tree_a.commit('Commit of rev2a', rev_id='rev2a')
         return tree_a, tree_b, branch_c
 
+    def test_empty_target(self):
+        tree_a, tree_b, branch_c = self.make_trees()
+        tree_d = self.make_branch_and_tree('tree_d')
+        md2 = merge_directive.MergeDirective.from_objects(
+            tree_a.branch.repository, 'rev2a', 500, 120, tree_d.branch.base,
+            patch_type='diff', public_branch=tree_a.branch.base)
+
     def test_generate_patch(self):
         tree_a, tree_b, branch_c = self.make_trees()
         md2 = merge_directive.MergeDirective.from_objects(

=== modified file 'bzrlib/tests/workingtree_implementations/test_merge_from_branch.py'
--- bzrlib/tests/workingtree_implementations/test_merge_from_branch.py	2007-03-10 21:39:07 +0000
+++ bzrlib/tests/workingtree_implementations/test_merge_from_branch.py	2007-06-19 14:49:06 +0000
@@ -19,6 +19,7 @@
 
 import os
 
+from bzrlib import errors
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 
 
@@ -66,3 +67,13 @@
         tree_a.lock_read()
         self.addCleanup(tree_a.unlock)
         list(tree_a._iter_changes(tree_a.basis_tree()))
+
+    def test_merge_empty(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        self.build_tree_contents([('tree_a/file', 'text-a')])
+        tree_a.add('file')
+        tree_a.commit('added file')
+        tree_b = self.make_branch_and_tree('treeb')
+        self.assertRaises(errors.NoCommits, tree_a.merge_from_branch,
+                          tree_b.branch)
+        tree_b.merge_from_branch(tree_a.branch)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-06-08 21:48:42 +0000
+++ bzrlib/workingtree.py	2007-06-19 14:49:06 +0000
@@ -813,7 +813,7 @@
                 to_revision = osutils.safe_revision_id(to_revision)
             merger.other_rev_id = to_revision
             if merger.other_rev_id is None:
-                raise error.NoCommits(branch)
+                raise errors.NoCommits(branch)
             self.branch.fetch(branch, last_revision=merger.other_rev_id)
             merger.other_basis = merger.other_rev_id
             merger.other_tree = self.branch.repository.revision_tree(

